/*
 * NAME:
 *	macros - miscellaneous macros
 *
 * DESCRIPTION:
 *	A collection of useful macros for C programers.
 *
 * AUTHOR:
 *	Simon J. Gerraty <sjg@quick.com.au>
 */
/*
 * RCSid:
 *	$Id: macros.h,v 1.2 1999/03/16 12:29:26 sjg Exp $
 *
 *	@(#)Copyright (c) 1998 QUICK.COM.AU. All rights reserved.
 *	@(#)Copyright (c) 1993 Simon J. Gerraty. All rights reserved.
 *	
 *	This file is provided in the hope that it will
 *	be of use.  There is absolutely NO WARRANTY.
 *	Permission to use, modify and distribute this source code 
 *	is granted subject to the following conditions.
 *	1/ that the above copyright notice and this notice 
 *	are preserved in all copies and that due credit be given 
 *	to the author.	
 *	2/ that any changes to this code are clearly commented 
 *	as such so that the author does not get blamed for bugs 
 *	other than his own.
 *	3/ If there is an accompanying License file its provisions
 *	must also be met.
 *	
 *	Please send copies of changes and bug-fixes to:
 *	sjg@quick.com.au
 */
#ifndef _macros_h
# define _macros_h

#ifndef NEW
# define NEW(type)	(type *) malloc(sizeof (type))
#endif
#ifndef MAX
# define MAX(a, b) (((a) < (b)) ? (b) : (a))
#endif
#ifndef MIN
# define MIN(a, b) (((a) < (b)) ? (a) : (b))
#endif
/*
 *	Count the number of u's in n
 */
#ifndef N2U
# define N2U(n, u)   (((n) + ((u) - 1)) / (u))
#endif
/*
 *	Round n up to the nearest u
 */
#ifndef ROUNDUP
# define ROUNDUP(n, u)   (N2U((n), (u)) * (u))
#endif
#ifndef ROUND_UP
# define ROUND_UP(n, u)   (N2U((n), (u)) * (u))
#endif

/*
 *	Use SkipWs() to avoid side effects.  There is TrimWs() too.
 */
#ifndef SKIP_WS
# define SKIP_WS(s)   while (s && (*s == '\n' || *s == '\r' || *s == ' ' || *s == '\t')) s++;
#endif

/*
 *      Load array "a" from "s" avoiding overrun.
 *	Returns "a".
 */
#ifndef ARRAY_STR
# define ARRAY_STR(a, s) \
        (strncpy((a), s, sizeof (a) - 1), (a)[sizeof (a) - 1] = '\0', (a))
#endif

/*
 *      Load "s" from array "a" and null terminate.
 *	Assumes "s" is at least as big as "a" and that "a" is null
 *	terminated (as though loaded by ARRAY_STR()).
 */
#ifndef STR_ARRAY
# define STR_ARRAY(s, a) \
        (strncpy(s, (a), sizeof (a) - 1), (s)[sizeof (a) - 1] = '\0', (s))
#endif
     
/*
 *      Concatenate string "s" to array "a".
 *      (assumes "a" has something sensible in it for strlen to work)
 */
#ifndef ARRAY_STRCAT
# define ARRAY_STRCAT(a, s) \
        (strncat((a), s, sizeof(a) - (strlen(a) + 1)), (a))
#endif
     
/*
 *      DANGER Will Robinson!
 *      If you are going to use this then "s" _must_ be
 *      at least 1 byte longer than "a".
 */
#ifndef STR_ARRAY_1
# define STR_ARRAY_1(s, a) \
        (strncpy((s), (a), sizeof(a)), (s)[sizeof(a)] = '\0')
#endif
     
#endif

